﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace iX.MeinWeinkeller.DAO
{
    /// <summary>
    /// WeinDAO stellt Create, Read, Update und Delete (CRUD) Operationen
    /// für Weine bereit
    /// </summary>
    public class WeinDAO
    {
        /// <summary>
        /// Erzeugt einen neuen Wein und gibt die neue id zurück
        /// </summary>
        /// <param name="strName">Name des Weins</param>
        /// <param name="strBeschreibung">Beschreibung des Weins</param>
        /// <param name="iJahrgang">Jahrgang des Weins</param>
        /// <param name="idWeingut">id des Weinguts</param>
        /// <returns>Die id des neuen Weins</returns>
        public static int Create(string strName, string strBeschreibung, int iJahrgang, decimal decPreis, int idWeingut)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Neue Instanz von Wein erzeugen und Eigenschaften setzen
            Wein wein = new Wein
            {
                Name = strName,
                Jahrgang = iJahrgang,
                Preis = decPreis,
                Beschreibung = strBeschreibung,
                WeingutID = idWeingut
            };
                
            // Dem Datenkontext signalisieren, dass der neue Datensatz eingefügt werden soll
            dc.Wein.InsertOnSubmit(wein);

            // Die Änderungen in der Datenbank speichern
            dc.SubmitChanges();

            // Die id des neuen Datensatzes zurückgeben
            return wein.id;
        }
        /// <summary>
        /// Liest einen einzelnen Wein
        /// </summary>
        /// <param name="id">id des Weins</param>
        /// <returns>Der Wein</returns>
        public static Wein ReadSingle(int id)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Einen einzelnen Wein lesen
            Wein Wein = dc.Wein.Single(w => w.id == id);

            // Den Wein zurückgeben
            return Wein;
        }

        /// <summary>
        /// Sucht nach einem String in ausgewählten Spalten der Tabelle Wein
        /// und gibt die gefundenen Weine zurück
        /// </summary>
        /// <param name="strSuchstring">Suchstring</param>
        /// <returns>Die selektierten Weine</returns>
        public static IQueryable<Wein> Read(string strSuchstring)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();
            
            // Weine gemäß Suchstring selektieren
            var weine =
                from w in dc.Wein
                where w.Name.ToLower().Contains(strSuchstring.ToLower()) ||
                      w.Beschreibung.Contains(strSuchstring.ToLower()) ||
                      w.Jahrgang.ToString().Contains(strSuchstring) ||
                      w.Weingut.Name.ToLower().Contains(strSuchstring.ToLower()) ||
                      w.Weingut.Ort.ToLower().Contains(strSuchstring.ToLower()) ||
                      w.Weingut.Nachname.ToLower().Contains(strSuchstring.ToLower())
                orderby w.Weingut.Ort, w.Weingut.Name, w.Name
                select w;

            // Weine zurückgeben
            return weine;
        }

        /// <summary>
        /// Gibt alle Weine zurück
        /// </summary>
        /// <returns>Alle Weine</returns>
        public static IQueryable<Wein> ReadAll()
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Alle Weine selektieren
            var weine =
                from w in dc.Wein
                orderby w.Name
                select w;

            // Weine zurückgeben
            return weine;
        }

        /// <summary>
        /// Ändert einen bestehenden Wein
        /// </summary>
        /// <param name="id">id des Weins</param>
        /// <param name="strName">Name des Weins</param>
        /// <param name="strBeschreibung">Beschreibung des Weins</param>
        /// <param name="iJahrgang">Jahrgang des Weins</param>
        /// <param name="idWeingut">id des Weinguts</param>
        /// <returns>Die id des neuen Weins</returns>
        public static void Update(int id, string strName, string strBeschreibung, int iJahrgang, decimal decPreis, int idWeingut)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Wein lesen
            Wein w = dc.Wein.Single(wg => wg.id == id);

            // Die geänderten Attribute setzen
            w.Name = strName;
            w.Jahrgang = iJahrgang;
            w.Preis = decPreis;
            w.Beschreibung = strBeschreibung;
            w.WeingutID = idWeingut;

            // Die Änderungen in der Datenbank speichern
            dc.SubmitChanges();
        }

        /// <summary>
        /// Löscht einen Wein 
        /// </summary>
        /// <param name="id">id des Weins</param>
        public static void Delete(int id)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Wein lesen
            Wein w = dc.Wein.Single(wg => wg.id == id);

            // Dem Datenkontext signalisieren, dass der Datensatz gelöscht werden soll
            dc.Wein.DeleteOnSubmit(w);
            FileDAO.DeleteFile(System.Web.HttpRuntime.AppDomainAppPath + "images\\wein", w.id);
            foreach (Weinlager wl in w.Weinlager)
                dc.Weinlager.DeleteOnSubmit(wl);

            // Den Datensatz löschen
            dc.SubmitChanges();
        }
    }
}
