﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace iX.MeinWeinkeller.DAO
{
    /// <summary>
    /// WeingutDAO stellt Create, Read, Update und Delete (CRUD) Operationen
    /// für Weingüter bereit
    /// </summary>
    public class WeingutDAO
    {
        /// <summary>
        /// Erzeugt ein neues Weingut und gibt die neue id zurück
        /// </summary>
        /// <param name="strName">Name des Weinguts</param>
        /// <param name="strBeschreibung">Beschreibung des Weinguts</param>
        /// <param name="strAnrede">Anrede des Besitzers</param>
        /// <param name="strVorname">Vorname des Besitzers</param>
        /// <param name="strNachname">Nachname des Besitzers</param>
        /// <param name="strStrasse">Strasse</param>
        /// <param name="strPostleitzahl">Postleitzahl</param>
        /// <param name="strOrt">Ort</param>
        /// <param name="strLand">Land</param>
        /// <returns>Die id des neuen Weinguts</returns>
        public static int Create(string strName, string strBeschreibung, string strAnrede, string strVorname, string strNachname, string strStrasse, string strPostleitzahl, string strOrt, string strLand)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Neue Instanz von Weingut erzeugen und Eigenschaften setzen
            Weingut weingut = new Weingut
            {
                Name = strName,
                Beschreibung = strBeschreibung,
                Anrede = strAnrede,
                Vorname = strVorname,
                Nachname = strNachname,
                Strasse = strStrasse,
                Postleitzahl = strPostleitzahl,
                Ort = strOrt,
                Land = strLand
            };

            // Dem Datenkontext signalisieren, dass der neue Datensatz eingefügt werden soll
            dc.Weingut.InsertOnSubmit(weingut);

            // Die Änderungen in der Datenbank speichern
            dc.SubmitChanges();

            // Die id des neuen Datensatzes zurückgeben
            return weingut.id;
        }
        /// <summary>
        /// Liest ein einzelnes Weingut
        /// </summary>
        /// <param name="id">id des Weinguts</param>
        /// <returns>Das Weingut</returns>
        public static Weingut ReadSingle(int id)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Ein einzelnes Weingut lesen
            Weingut weingut = dc.Weingut.Single(wg => wg.id == id);

            // Das Weingut zurückgeben
            return weingut;
        }

        /// <summary>
        /// Sucht nach einem String in ausgewählten Spalten der Tabelle Weingut
        /// und gibt die gefundenen Weingüter zurück
        /// </summary>
        /// <param name="strSuchstring">Suchstring</param>
        /// <returns>Die selektierten Weingüter</returns>
        public static IQueryable<Weingut> Read(string strSuchstring)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Weingüter gemäß Suchstring selektieren
            var weingueter =
                from wg in dc.Weingut
                where wg.Name.ToLower().Contains(strSuchstring.ToLower()) ||
                      wg.Land.ToLower().Contains(strSuchstring.ToLower()) ||
                      wg.Ort.ToLower().Contains(strSuchstring.ToLower()) ||
                      wg.Nachname.ToLower().Contains(strSuchstring.ToLower()) ||
                      wg.Vorname.ToLower().Contains(strSuchstring.ToLower())
                orderby wg.Name
                select wg;

            // Weingüter zurückgeben
            return weingueter;
        }

        /// <summary>
        /// Gibt alle Weingüter zurück
        /// </summary>
        /// <returns>Alle Weingüter</returns>
        public static IQueryable<Weingut> ReadAll()
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Alle Weingüter selektieren
            var weingueter =
                from wg in dc.Weingut
                orderby wg.Name
                select wg;

            // Weingüter zurückgeben
            return weingueter;
        }

        /// <summary>
        /// Ändert ein bestehendes Weingut
        /// </summary>
        /// <param name="id">id des Weinguts</param>
        /// <param name="strName">Name des Weinguts</param>
        /// <param name="strBeschreibung">Beschreibung des Weinguts</param>
        /// <param name="strAnrede">Anrede des Besitzers</param>
        /// <param name="strVorname">Vorname des Besitzers</param>
        /// <param name="strNachname">Nachname des Besitzers</param>
        /// <param name="strStrasse">Strasse</param>
        /// <param name="strPostleitzahl">Postleitzahl</param>
        /// <param name="strOrt">Ort</param>
        /// <param name="strLand">Land</param>
        public static void Update(int id, string strName, string strBeschreibung, string strAnrede, string strVorname, string strNachname, string strStrasse, string strPostleitzahl, string strOrt, string strLand)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Weingut lesen
            Weingut weingut = dc.Weingut.Single(wg => wg.id == id);

            // Die geänderten Attribute setzen
            weingut.Name = strName;
            weingut.Beschreibung = strBeschreibung;
            weingut.Anrede = strAnrede;
            weingut.Vorname = strVorname;
            weingut.Nachname = strNachname;
            weingut.Strasse = strStrasse;
            weingut.Postleitzahl = strPostleitzahl;
            weingut.Ort = strOrt;
            weingut.Land = strLand;

            // Die Änderungen in der Datenbank speichern
            dc.SubmitChanges();
        }

        /// <summary>
        /// Löscht ein Weingut 
        /// </summary>
        /// <param name="id">id des Weinguts</param>
        public static void Delete(int id)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Weingut lesen
            Weingut weingut = dc.Weingut.Single(wg => wg.id == id);

            // Dem Datenkontext signalisieren, dass der Datensatz gelöscht werden soll
            dc.Weingut.DeleteOnSubmit(weingut);
            FileDAO.DeleteFile(System.Web.HttpRuntime.AppDomainAppPath + "images\\weingut", weingut.id);
            foreach (Wein wein in weingut.Wein)
            {
                dc.Wein.DeleteOnSubmit(wein);
                FileDAO.DeleteFile(System.Web.HttpRuntime.AppDomainAppPath + "images\\wein", wein.id);
                foreach (Weinlager weinlager in wein.Weinlager)
                    dc.Weinlager.DeleteOnSubmit(weinlager);
            }

            // Den Datensatz löschen
            dc.SubmitChanges();
        }
    }
}
