﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace iX.MeinWeinkeller.DAO
{
    /// <summary>
    /// WeinkellerDAO stellt Create, Read, Update und Delete (CRUD) Operationen
    /// für Weinkeller bereit
    /// </summary>
    public class WeinkellerDAO
    {
        /// <summary>
        /// Erzeugt einen neuen Weinkeller und gibt die neue id zurück
        /// </summary>
        /// <param name="strName">Name des Weinkellers</param>
        /// <param name="strOrt">Ort des Weinkellers</param>
        /// <returns>Die id des neuen Weinkellers</returns>
        public static int Create(string strName, string strOrt)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Neue Instanz von Weinkeller erzeugen und Eigenschaften setzen
            Weinkeller weinkeller = new Weinkeller
            {
                Name = strName,
                Ort = strOrt
            };

            // Dem Datenkontext signalisieren, dass der neue Datensatz eingefügt werden soll
            dc.Weinkeller.InsertOnSubmit(weinkeller);

            // Die Änderungen in der Datenbank speichern
            dc.SubmitChanges();

            // Die id des neuen Datensatzes zurückgeben
            return weinkeller.id;
        }
        /// <summary>
        /// Liest einen einzelnen Weinkeller
        /// </summary>
        /// <param name="id">id des Weinkellers</param>
        /// <returns>Der Weinkeller</returns>
        public static Weinkeller ReadSingle(int id)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Einen einzelnen Weinkeller lesen
            Weinkeller weinkeller = dc.Weinkeller.Single(w => w.id == id);

            // Den Weinkeller zurückgeben
            return weinkeller;
        }

        /// <summary>
        /// Sucht nach einem String in ausgewählten Spalten der Tabelle Weinkeller
        /// und gibt die gefundenen Weinkeller zurück
        /// </summary>
        /// <param name="strSuchstring">Suchstring</param>
        /// <returns>Die selektierten Weinkeller</returns>
        public static IQueryable<Weinkeller> Read(string strSuchstring)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Weinkeller gemäß Suchstring selektieren
            var weinkeller =
                from wk in dc.Weinkeller
                where wk.Name.ToLower().Contains(strSuchstring.ToLower()) ||
                      wk.Ort.ToLower().Contains(strSuchstring.ToLower())
                orderby wk.Name
                select wk;

            // Weinkeller zurückgeben
            return weinkeller;
        }

        /// <summary>
        /// Gibt alle Weinkeller zurück
        /// </summary>
        /// <returns>Alle Weinkeller</returns>
        public static IQueryable<Weinkeller> ReadAll()
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Alle Weinkeller selektieren
            var weinkeller =
                from wk in dc.Weinkeller
                orderby wk.Name
                select wk;

            // Weinkeller zurückgeben
            return weinkeller;
        }

        /// <summary>
        /// Gibt Weinkeller zurück, in denen ein Wein noch nicht lagert
        /// </summary>
        /// <param name="idWein">id des Weins</param>
        /// <returns>Weinkeller, in denen der Wein nicht lagert</returns>
        public static IQueryable<Weinkeller> ReadEligible(int idWein)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();
            
            var weinkeller =
                from wk in dc.Weinkeller
                let count = 
                  dc.Weinlager.Count(wl => wl.WeinkellerID == wk.id && wl.WeinID == idWein)                  
                where count == 0
                select wk;

            return weinkeller;
        }

        /// <summary>
        /// Ändert einen bestehenden Weinkeller
        /// </summary>
        /// <param name="id">id des Weinkellers</param>
        /// <param name="strName">Name des Weinkellers</param>
        /// <param name="strOrt">Ort des Weinkellers</param>
        /// <returns>Die id des neuen Weinkellers</returns>
        public static void Update(int id, string strName, string strOrt)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Weinkeller lesen
            Weinkeller wk = dc.Weinkeller.Single(w => w.id == id);

            // Die geänderten Attribute setzen
            wk.Name = strName;
            wk.Ort = strOrt;

            // Die Änderungen in der Datenbank speichern
            dc.SubmitChanges();
        }

        /// <summary>
        /// Löscht einen Weinkeller 
        /// </summary>
        /// <param name="id">id des Weinkellers</param>
        public static void Delete(int id)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Neue Instanz von Weinkeller erzeugen und id setzen
            Weinkeller weinkeller = dc.Weinkeller.Single(w => w.id == id);

            // Dem Datenkontext signalisieren, dass der Datensatz gelöscht werden soll
            dc.Weinkeller.DeleteOnSubmit(weinkeller);

            FileDAO.DeleteFile(System.Web.HttpRuntime.AppDomainAppPath + "images\\weinkeller", weinkeller.id);
            foreach (Weinlager wl in weinkeller.Weinlager)
                dc.Weinlager.DeleteOnSubmit(wl);

            // Den Datensatz löschen
            dc.SubmitChanges();
        }
    }
}
