﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace iX.MeinWeinkeller.DAO
{
    /// <summary>
    /// WeinlagerDAO stellt Create, Read, Update und Delete (CRUD) Operationen
    /// für Weinlager bereit
    /// </summary>
    public class WeinlagerDAO
    {
        /// <summary>
        /// Erzeugt ein neues Weinlager und gibt die neue id zurück
        /// </summary>
        /// <param name="idWeinkeller">id des Weinkellers</param>
        /// <param name="idWein">id des Weins</param>
        /// <param name="iFlaschenzahl">Anzahl der Flaschen</param>
        /// <returns>Die id des neuen Weinlagers</returns>
        public static int Create(int idWeinkeller, int idWein, int iFlaschenzahl)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Neue Instanz von Weinlager erzeugen und Eigenschaften setzen
            Weinlager weinlager = new Weinlager
            {
                WeinkellerID = idWeinkeller,
                WeinID = idWein,
                Flaschenzahl = iFlaschenzahl
            };

            // Dem Datenkontext signalisieren, dass der neue Datensatz eingefügt werden soll
            dc.Weinlager.InsertOnSubmit(weinlager);

            // Die Änderungen in der Datenbank speichern
            dc.SubmitChanges();

            // Die id des neuen Datensatzes zurückgeben
            return weinlager.id;
        }
        /// <summary>
        /// Liest ein einzelnes Weinlager
        /// </summary>
        /// <param name="id">id des Weinlagers</param>
        /// <returns>Das Weinlager</returns>
        public static Weinlager ReadSingle(int id)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Ein einzelnes Weinlager lesen
            Weinlager weinlager = dc.Weinlager.Single(wl => wl.id == id);

            // Das Weinlager zurückgeben
            return weinlager;
        }

        /// <summary>
        /// Gibt alle Weinlager eines Weins zurück
        /// </summary>
        /// <param name="idWein">id des Weins</param>
        /// <returns>Alle Weinlager dieses Weins</returns>
        public static IQueryable<Weinlager> ReadByWein(int idWein)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Alle Weinlager eines Weins selektieren
            var weinlager =
                from wl in dc.Weinlager
                where wl.WeinID == idWein
                select wl;

            // Weinlager zurückgeben
            return weinlager;
        }

        /// <summary>
        /// Gibt alle Weinlager eines Weinkellers zurück
        /// </summary>
        /// <param name="idWeinkeller">id des Weinkellers</param>
        /// <returns>Alle Weinlager dieses Weinkellers</returns>
        public static IQueryable<Weinlager> ReadByWeinkeller(int idWeinkeller)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Alle Weinlager eines Weinkellers selektieren
            var weinlager =
                from wl in dc.Weinlager
                where wl.WeinkellerID == idWeinkeller
                select wl;

            // Weinlager zurückgeben
            return weinlager;
        }

        /// <summary>
        /// Ändert ein bestehendes Weinlager
        /// </summary>
        /// <param name="id">id des Weinlagers</param>
        /// <param name="idWeinkeller">id des Weinkellers</param>
        /// <param name="idWein">id des Weins</param>
        /// <param name="iFlaschenzahl">Anzahl der Flaschen</param>
        /// <returns>Die id des neuen Weinlagers</returns>
        public static void Update(int id, int iFlaschenzahl)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Weinlager lesen
            Weinlager weinlager = dc.Weinlager.Single(wl => wl.id == id);

            // Die geänderten Attribute setzen
            weinlager.Flaschenzahl = iFlaschenzahl;

            // Die Änderungen in der Datenbank speichern
            dc.SubmitChanges();
        }

        /// <summary>
        /// Löscht ein Weinlager 
        /// </summary>
        /// <param name="id">id des Weinlagers</param>
        public static void Delete(int id)
        {
            // Eine Instanz des Datenkontextes erzeugen
            DatenKlassenDataContext dc = new DatenKlassenDataContext();

            // Neue Instanz von Weinlager erzeugen und id setzen
            Weinlager weinlager = dc.Weinlager.Single(wl => wl.id == id);

            // Dem Datenkontext signalisieren, dass der Datensatz gelöscht werden soll
            dc.Weinlager.DeleteOnSubmit(weinlager);

            // Den Datensatz löschen
            dc.SubmitChanges();
        }
    }
}
